package com.sonicsw.psdn.jms.samples.loadBalancedConnection;

import javax.jms.Connection;


import progress.message.jclient.ConnectionFactory;

/**
 * This is a simple program to show how to produce messages to either a queue or a topic, using the
 * latest JMS API.
 * @author dmillman
 *
 */
public class LoadBalancedConnection {
	
	
	/**
	 * Initialize the Queue send object so that it can send messages, this method does not send any messages
	 * but ensures that everything is initialized for the messages to be sent.
	 * @param argURL the URL of the acceptor on the broker to connect to.
	 * @param argUsername the name of the user to connect as.
	 * @param argPassword the password of the user to connect as.
	 * @param argDestinationName the name of the destination to create
	 * @param argDestinationType the type of destination to create
	 * @throws Exception an exception is thrown if any of the steps fails.
	 */
	public void initialize( String argURL, String argUsername, String argPassword) throws Exception{
		this.initializeConnectionFactory(argURL,argUsername,argPassword);
		this.initializeConnection();
		this.connection_.start();
	}
	
	/**
	 * 
	 * @param argURL
	 * @param argUsername
	 * @param argPassword
	 * @throws Exception
	 */
	private void initializeConnectionFactory( String argURL, String argUsername, String argPassword ) throws Exception{
		this.connectionFactory_ = new ConnectionFactory();
		// Set load balancing on, so that the connection will be load balanced across the cluster
		this.connectionFactory_.setLoadBalancing(LOAD_BALANCED);
		this.connectionFactory_.setConnectionURLs(argURL);
		this.connectionFactory_.setDefaultUser(argUsername);
		this.connectionFactory_.setDefaultPassword(argPassword);
		
		this.connectionFactory_.setFaultTolerant(new Boolean(true));
	}
	
	/**
	 * 
	 * @throws Exception
	 */
	private void initializeConnection() throws Exception{
		this.connection_ = this.connectionFactory_.createConnection();
		progress.message.jclient.Connection connection = (progress.message.jclient.Connection)this.connection_;
		System.out.println("Broker URL = " + connection.getBrokerURL());
	}
	

	
	
	/**
	 * These attributes maintain access to the various JMS objects that are created.
	 */
	private ConnectionFactory connectionFactory_;
	private Connection        connection_;
	
	
	//
	// Constant(s)
	//
	private static final boolean LOAD_BALANCED                = true;

	
	/**
	 * Main routine that is executed to produce messages to the specified destination.
	 * @param args
	 */
	public static void main(String[] args) {
		try{
			String URL                  = args[0];
			String username             = args[1];
			String password             = args[2];
			int    connections          = Integer.parseInt(args[3]);
			
			// Echo the parameters that were passed in back to the user.
			System.out.println("URL                    = " + URL);
			System.out.println("Username               = " + username);
			System.out.println("Password               = " + password);
			System.out.println("Number of Connections  = " + connections);
			
			
			// Initialize the connections etc and then send the messages, finally destroy
			// the connection to the broker.
			LoadBalancedConnection[] lbConnections = new LoadBalancedConnection[connections];
			for ( int i=0; i < connections; i++){
				lbConnections[i] = new LoadBalancedConnection();
				lbConnections[i].initialize(URL,username,password);
			}
		}
		catch ( Exception e){
			e.printStackTrace();
		}
	}

}
